/*
 * A memory pool providing regions from a preallocated amount of memory.
 *
 * Copyright (C) 2021 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include "memory_preallocated.h"



/* Initialise the memory pool with the given amount in bytes. */

MemoryPreallocated::MemoryPreallocated(Memory *memory, offset_t amount)
: _memory(memory), _amount(amount)
{
    // NOTE: Handle allocation failure.

    offset_t remaining = amount;

    while (remaining)
    {
        _regions.push_back(memory->region());

        if (memory->region_size() >= remaining)
            break;

        remaining -= memory->region_size();
    }
}

/* Discard all unused regions. */

MemoryPreallocated::~MemoryPreallocated()
{
    while (!_regions.empty())
    {
        Region *region = _regions.front();

        _regions.pop_front();
        _memory->release(region);
    }
}

/* Obtain an allocated region. */

Region *MemoryPreallocated::region()
{
    std::lock_guard<std::mutex> guard(_lock);

    if (_regions.empty())
        return NULL;

    Region *region = _regions.front();

    _regions.pop_front();
    return region;
}

/* Release the allocated 'region'. */

void MemoryPreallocated::release(Region *region)
{
    std::lock_guard<std::mutex> guard(_lock);

    _regions.push_back(region);
}

// vim: tabstop=4 expandtab shiftwidth=4
